import { EventEmitter } from 'events';
import { Sleep } from './sleep';

export interface ITask {
    delegate: Delegate;
    title: string;
}

export type Delegate = () => Promise<void>;

export class Scheduler extends EventEmitter {
    private _tasks: Array<ITask>;
    private _isStarted: boolean;
    private _log: any;
    private _delay: number;
    private _watchdog: number;

    constructor(delay: number, watchdog: number, log: any) {
        super();
        this._tasks = [];
        this._isStarted = false;
        this._log = log;
        this._delay = delay;
        this._watchdog = watchdog;
    }

    public get IsStarted(): boolean {
        return this._isStarted;
    }

    public addTask = (task: ITask): Scheduler => {
        if (this._isStarted) {
            throw new Error("Stop the scheduler before adding a task");
        }

        this._tasks.push(task);

        return this;
    }

    public deleteTask = (task: ITask): Scheduler => {
        if (this._isStarted) {
            throw new Error("Stop the scheduler before removing a task");
        }

        const idx = this._tasks.indexOf(task);
        if (idx > -1) {
            this._tasks.splice(idx, 1);
        }

        return this;
    }

    public start = (): void => {
        if (this._tasks.length > 0 && !this._isStarted) {
            this.begin();
        }
    }

    public stop = (): void => {
        if (this._isStarted) {
            this._isStarted = false;
        }
    }

    private begin = async (): Promise<void> => {
        let current = "";

        const timeoutId = setTimeout(() => {
            this.emit('error', new Error(`Task '${current}' failed to execute within the watchdog timer`));
        },
            this._watchdog);

        timeoutId.unref();

        this._isStarted = true;
        while (this._isStarted) {
            for (const task of this._tasks) {
                try {
                    timeoutId.refresh();
                    current = task.title;

                    await task.delegate();
                } catch (err) {
                    this.emit('error', err);
                    continue;
                }
            }

            await Sleep(this._delay);
        }
    }
}